# -*- coding: utf-8 -*-
from libs import management
from helpers.encryption import TwoWayEncryption
from helpers.synchronizing import synchronous
from helpers.caching import CachedMethod
from constants import BACKUP_RESULT_HISTORY_TIME
import uuid, datetime, sqlite3, threading, json


class BackupManager(object):

    def __init__(self, host, port):

        self._host = host
        self._port = port
        self._conn = sqlite3.connect(':memory:')
        self._encryptions = dict()
        self.initialize_db()
        self.progress_lock = threading.RLock()

    def initialize_db(self):

        c = self._conn.cursor()
        c.execute("""CREATE TABLE backups(
            id TEXT PRIMARY KEY,
            namespace TEXT,
            destination_node_address TEXT,
            username TEXT,
            password TEXT,
            destination_location TEXT,
            sets TEXT,
            only_metadata TEXT,
            progress_file TEXT,
            delete_it INT,
            timestamp TIMESTAMP,
            status TEXT)""")
        self._conn.commit()

    def add_backup(self, backup_id, namespace, destination_node_address, username,
         password, destination_location, sets, only_metadata, progress_file):

        if username != "" and password is not None:
            self._encryptions[backup_id] = TwoWayEncryption()
            username = self._encryptions[backup_id].encrypt(username)
            password = self._encryptions[backup_id].encrypt(password)

        c = self._conn.cursor()
        c.execute("""INSERT INTO backups VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)""",
            (backup_id, namespace, destination_node_address, username,
            password, destination_location, sets, only_metadata, progress_file, 0, None, None))

    def get_backups(self):

        c = self._conn.cursor()
        c.execute("SELECT * FROM backups")
        return c.fetchall()

    def check_if_backup_exists(self, namespace, destination_node_address,
         destination_location, sets, only_metadata):

        c = self._conn.cursor()
        c.execute("SELECT id, count(id) FROM backups WHERE namespace like ? and destination_node_address like ? and destination_location like ? and sets like ? and only_metadata like ? and delete_it not like ?", (namespace, destination_node_address, destination_location, sets, only_metadata, 1))
        return c.fetchone()

    def delete_backup(self, backup_id):

        c = self._conn.cursor()
        c.execute("DELETE FROM backups WHERE id like ?", (backup_id, ))

    def mark_backup_to_delete_it(self, backup_id, status):

        c = self._conn.cursor()
        c.execute("UPDATE backups SET delete_it = 1, timestamp = ?, status = ? WHERE id like ?", (datetime.datetime.now().strftime("%Y/%m/%d %H:%M:%S"), status, backup_id))

    def initiate_backup(self, namespace, destination_node_address, username,
    password, destination_location, aerospike_build, sets="ALL",
    only_metadata="False", scan_priority=2, terminate_on_change="True"):

        backup_check = self.check_if_backup_exists(namespace, destination_node_address,
        destination_location, sets, only_metadata)
 
        if backup_check[1] != 0:
            return dict(status="Failure", error="Backup already initiated", id=backup_check[0])

        optional_args = ""
        if sets != "ALL" and sets != "":
            optional_args += "-s %s " % sets
        if only_metadata == "True":
            optional_args += "-x "
        if terminate_on_change == "True":
            optional_args += "-c "
        if scan_priority != "2":
            optional_args += "-f %s " % str(scan_priority)

        backup_id = str(uuid.uuid4())
        progress_file = "/tmp/backup-" + backup_id + ".log"

        timestamp = datetime.datetime.now().strftime("%Y-%m-%d_%H:%M:%S")
        destination_location = destination_location.rstrip("/") + "/backup_" + timestamp

        if aerospike_build.startswith('3'):
            cmd = "/bin/bash -c 'mkdir -p %s && asbackup -h %s -p %s -d %s -n %s %s &> %s;'" % (destination_location, self._host, self._port, destination_location, namespace, optional_args, progress_file)
        else:
            cmd = "/bin/bash -c 'mkdir -p %s && clbackup2 -h %s -p %s -d %s -n %s %s &> %s;'" % (destination_location, self._host, self._port, destination_location, namespace, optional_args, progress_file)

        result = management.handle_remote_command(destination_node_address,
        username, password, cmd, background=3600, backup_flag=True,
        module_name="shell")

        if result['status'] != "Failure":
            self.add_backup(backup_id, namespace, destination_node_address,
            username, password, destination_location, sets, only_metadata,
            progress_file)
            result.update(backup_id=backup_id)
        return result

    @synchronous('progress_lock')
    @CachedMethod()
    def get_backup_progress(self, cluster_level_alerts):

        backups = self.get_backups()
        response = dict()
   
        for backup in backups:
            response[backup[0]] = dict()
            response[backup[0]]['namespace'] = backup[1]
            response[backup[0]]['destination_node_address'] = backup[2]
            response[backup[0]]['destination_location'] = backup[5]
            if backup[9] == 1:
                response[backup[0]]['progress'] = json.loads(backup[11])
                timestamp = datetime.datetime.now() - datetime.timedelta(seconds=BACKUP_RESULT_HISTORY_TIME)
                timestamp = timestamp.strftime("%Y/%m/%d %H:%M:%S")
                if timestamp >= backup[10]:
                    self.delete_backup(backup[0])
            else:
                username = backup[3]
                password = backup[4]
                if username != "" and password is not None:
                    username = self._encryptions[backup[0]].decrypt(username)
                    password = self._encryptions[backup[0]].decrypt(password)
                response[backup[0]]['progress'] = management.get_backup_progress(
                    backup[2], username, password, backup[8])
                if response[backup[0]]['progress']['status'] == "Success" or response[backup[0]]['progress']['status'] == "Failure":
                    self.mark_backup_to_delete_it(backup[0], json.dumps(response[backup[0]]['progress']))
                    if backup[0] in list(self._encryptions.keys()):
                        del self._encryptions[backup[0]]
                if response[backup[0]]['progress']['status'] == "Success":
                    cluster_level_alerts.on_backup_success(backup[1], backup[2],
                    backup[5])     
        return response

